/*
 * Copyright (C) 2015 Freescale Semiconductor, Inc.
 * Copyright (C) 2016 Atmark Techno, Inc.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <asm/arch/clock.h>
#include <asm/arch/imx-regs.h>
#include <asm/arch/mx7-pins.h>
#include <asm/arch/sys_proto.h>
#include <asm/gpio.h>
#include <asm/imx-common/iomux-v3.h>
#include <asm/imx-common/boot_mode.h>
#include <asm/io.h>
#include <linux/sizes.h>
#include <common.h>
#include <fsl_esdhc.h>
#include <mmc.h>
#include <miiphy.h>
#include <netdev.h>
#ifdef CONFIG_SYS_I2C_MXC
#include <i2c.h>
#include <asm/imx-common/mxc_i2c.h>
#endif
#include <asm/arch/crm_regs.h>
#include <watchdog.h>

#ifdef CONFIG_FASTBOOT
#include <fastboot.h>
#endif /*CONFIG_FASTBOOT*/


DECLARE_GLOBAL_DATA_PTR;

#define UART_PAD_CTRL  (PAD_CTL_DSE_3P3V_49OHM | \
	PAD_CTL_PUS_PU100KOHM | PAD_CTL_HYS)

#define USDHC_PAD_CTRL (PAD_CTL_DSE_3P3V_32OHM | PAD_CTL_SRE_SLOW | \
	PAD_CTL_HYS | PAD_CTL_PUE | PAD_CTL_PUS_PU47KOHM)
#define USDHC_PAD_CTRL_CD (PAD_CTL_DSE_3P3V_32OHM | PAD_CTL_SRE_SLOW | \
	PAD_CTL_HYS)
#define USDHC_PAD_CTRL_WP (PAD_CTL_DSE_3P3V_32OHM | PAD_CTL_SRE_SLOW | \
	PAD_CTL_HYS)

#define ENET_PAD_CTRL  (PAD_CTL_PUS_PU100KOHM | PAD_CTL_DSE_3P3V_49OHM)
#define ENET_PAD_CTRL_MII  (PAD_CTL_DSE_3P3V_32OHM)

#define ENET_RX_PAD_CTRL  (PAD_CTL_PUS_PU100KOHM | PAD_CTL_DSE_3P3V_49OHM)

#define I2C_PAD_CTRL    0x0

#define QSPI_PAD_CTRL	\
	(PAD_CTL_DSE_3P3V_49OHM | PAD_CTL_PUE | PAD_CTL_PUS_PU47KOHM)

#define SPI_PAD_CTRL (PAD_CTL_DSE_3P3V_49OHM | PAD_CTL_SRE_SLOW | PAD_CTL_HYS)

#define GPIO_PAD_CTRL (PAD_CTL_SRE_SLOW | PAD_CTL_PUE | PAD_CTL_PUS_PD100KOHM)

#ifdef CONFIG_SYS_I2C_MXC
#define PC MUX_PAD_CTRL(I2C_PAD_CTRL)

#define BOARDTYPE_AIOTG_G3  0x0A000000
#define BOARDTYPE_AX1       0x0A100000
#define BOARDTYPE_AIOTG_G3L 0x0A200000

struct board_info {
	unsigned int board_type;
	unsigned short revison;
	unsigned int serial_no;
	unsigned int dram;
	unsigned char xtal;
	unsigned char usermac[6];
};

static struct board_info x1_board_info;

/* I2C4 for PMIC */
struct i2c_pads_info i2c_pad_info4 = {
	.scl = {
		.i2c_mode = MX7D_PAD_GPIO1_IO10__I2C4_SCL | PC,
		.gpio_mode = MX7D_PAD_GPIO1_IO10__GPIO1_IO10 | PC,
		.gp = IMX_GPIO_NR(1, 10),
	},
	.sda = {
		.i2c_mode = MX7D_PAD_GPIO1_IO11__I2C4_SDA | PC,
		.gpio_mode = MX7D_PAD_GPIO1_IO11__GPIO1_IO11 | PC,
		.gp = IMX_GPIO_NR(1, 11),
	},
};
#endif

int dram_init(void)
{
	gd->ram_size = PHYS_SDRAM_SIZE;

	return 0;
}

static iomux_v3_cfg_t const wdog_pads[] = {
	MX7D_PAD_GPIO1_IO00__WDOG1_WDOG_B | MUX_PAD_CTRL(NO_PAD_CTRL),
};

static iomux_v3_cfg_t const uart5_pads[] = {
	MX7D_PAD_GPIO1_IO06__UART5_RX_DATA | MUX_PAD_CTRL(UART_PAD_CTRL),
	MX7D_PAD_GPIO1_IO07__UART5_TX_DATA | MUX_PAD_CTRL(UART_PAD_CTRL),
	MX7D_PAD_GPIO1_IO04__UART5_CTS_B | MUX_PAD_CTRL(UART_PAD_CTRL),
	MX7D_PAD_GPIO1_IO05__UART5_RTS_B | MUX_PAD_CTRL(UART_PAD_CTRL),
};

static iomux_v3_cfg_t const usdhc1_pads[] = {
	MX7D_PAD_SD1_CLK__SD1_CLK | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD1_CMD__SD1_CMD | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD1_DATA0__SD1_DATA0 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD1_DATA1__SD1_DATA1 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD1_DATA2__SD1_DATA2 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD1_DATA3__SD1_DATA3 | MUX_PAD_CTRL(USDHC_PAD_CTRL),

	MX7D_PAD_SD1_CD_B__GPIO5_IO0 | MUX_PAD_CTRL(USDHC_PAD_CTRL_CD),
	MX7D_PAD_SD1_WP__GPIO5_IO1 | MUX_PAD_CTRL(USDHC_PAD_CTRL_WP),
	MX7D_PAD_SD1_RESET_B__GPIO5_IO2 | MUX_PAD_CTRL(NO_PAD_CTRL),
};

static iomux_v3_cfg_t const usdhc3_emmc_pads[] = {
	MX7D_PAD_SD3_CLK__SD3_CLK | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD3_CMD__SD3_CMD | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD3_DATA0__SD3_DATA0 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD3_DATA1__SD3_DATA1 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD3_DATA2__SD3_DATA2 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD3_DATA3__SD3_DATA3 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD3_DATA4__SD3_DATA4 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD3_DATA5__SD3_DATA5 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD3_DATA6__SD3_DATA6 | MUX_PAD_CTRL(USDHC_PAD_CTRL),
	MX7D_PAD_SD3_DATA7__SD3_DATA7 | MUX_PAD_CTRL(USDHC_PAD_CTRL),

	MX7D_PAD_SD3_RESET_B__GPIO6_IO11 | MUX_PAD_CTRL(NO_PAD_CTRL),
};

static iomux_v3_cfg_t const gpio_aiot_g3_led_pads[] = {
	MX7D_PAD_LCD_DATA06__GPIO3_IO11 | MUX_PAD_CTRL(NO_PAD_CTRL),
	MX7D_PAD_LCD_DATA05__GPIO3_IO10 | MUX_PAD_CTRL(NO_PAD_CTRL),
	MX7D_PAD_LCD_DATA07__GPIO3_IO12 | MUX_PAD_CTRL(NO_PAD_CTRL),
	MX7D_PAD_LCD_DATA08__GPIO3_IO13 | MUX_PAD_CTRL(NO_PAD_CTRL),
};

static iomux_v3_cfg_t const gpio_aiot_g3_addon_pads[] = {
	MX7D_PAD_LCD_CLK__GPIO3_IO0 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_3, CON2_41 */
	MX7D_PAD_LCD_ENABLE__GPIO3_IO1 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_4, CON2_40 */
	MX7D_PAD_LCD_HSYNC__GPIO3_IO2 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_5, CON2_39 */
	MX7D_PAD_LCD_VSYNC__GPIO3_IO3 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_6, CON2_38 */
	MX7D_PAD_UART2_RX_DATA__GPIO4_IO2 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_7, CON2_32 */
	MX7D_PAD_UART2_TX_DATA__GPIO4_IO3 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_8, CON2_34 */
	MX7D_PAD_UART3_RX_DATA__GPIO4_IO4 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_9, CON2_36 */
	MX7D_PAD_UART3_TX_DATA__GPIO4_IO5 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_10, CON2_37 */
	MX7D_PAD_UART3_RTS_B__GPIO4_IO6 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_11, CON2_35 */
	MX7D_PAD_UART3_CTS_B__GPIO4_IO7 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_12, CON2_50 */
	MX7D_PAD_I2C2_SCL__GPIO4_IO10 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_13, CON2_24 */
	MX7D_PAD_I2C2_SDA__GPIO4_IO11 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_14, CON2_25 */
	MX7D_PAD_SD2_RESET_B__GPIO5_IO11 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_15, CON2_33 */
	MX7D_PAD_SAI2_TX_SYNC__GPIO6_IO19 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_16, CON2_46 */
	MX7D_PAD_SAI2_TX_BCLK__GPIO6_IO20 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_17, CON2_47 */
	MX7D_PAD_SAI2_TX_DATA__GPIO6_IO22 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_18, CON2_49 */
	MX7D_PAD_SAI2_RX_DATA__GPIO6_IO21 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_19, CON2_48 */
	MX7D_PAD_I2C3_SCL__GPIO4_IO12 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_22, CON2_42 */
	MX7D_PAD_I2C3_SDA__GPIO4_IO13 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_23, CON2_43 */
	MX7D_PAD_LCD_DATA20__GPIO3_IO25 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_24 */
	MX7D_PAD_LCD_DATA21__GPIO3_IO26 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_25 */
	MX7D_PAD_ENET1_TX_CLK__GPIO7_IO12 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_32 */
	MX7D_PAD_ENET1_RX_CLK__GPIO7_IO13 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_33 */
	MX7D_PAD_ENET1_RGMII_TD1__GPIO7_IO7 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_34 */
	MX7D_PAD_ENET1_RGMII_RD2__GPIO7_IO2 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_35 */
	MX7D_PAD_ENET1_RGMII_TD2__GPIO7_IO8 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_36 */
	MX7D_PAD_ENET1_RGMII_RD3__GPIO7_IO3 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_37 */
	MX7D_PAD_ENET1_RGMII_RD0__GPIO7_IO0 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_38 */
	MX7D_PAD_ENET1_RGMII_RD1__GPIO7_IO1 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_39 */
	MX7D_PAD_UART1_TX_DATA__GPIO4_IO1 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_40 */
	MX7D_PAD_UART1_RX_DATA__GPIO4_IO0 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_41 */
	MX7D_PAD_ENET1_RGMII_RX_CTL__GPIO7_IO4 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_42 */
	MX7D_PAD_ENET1_RGMII_RXC__GPIO7_IO5 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_43 */
	MX7D_PAD_ENET1_RGMII_TD0__GPIO7_IO6 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_44 */
	MX7D_PAD_ENET1_RGMII_TX_CTL__GPIO7_IO10 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_45 */
	MX7D_PAD_ENET1_RGMII_TXC__GPIO7_IO11 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_46 */
	MX7D_PAD_SD2_CLK__GPIO5_IO12 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_47 */
	MX7D_PAD_SD2_CMD__GPIO5_IO13 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_48 */
	MX7D_PAD_SD2_DATA0__GPIO5_IO14 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_49 */
	MX7D_PAD_ENET1_RGMII_TD3__GPIO7_IO9 | MUX_PAD_CTRL(GPIO_PAD_CTRL),	/* CON1_50 */
	MX7D_PAD_SD2_DATA1__GPIO5_IO15 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_51 */
	MX7D_PAD_SD2_DATA2__GPIO5_IO16 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_52 */
	MX7D_PAD_SD2_DATA3__GPIO5_IO17 | MUX_PAD_CTRL(GPIO_PAD_CTRL),		/* CON1_53 */
};

static const u32 gpio_aiot_g3_addon[] = {
	IMX_GPIO_NR(3, 0),	/* CON1_3, CON2_41 */
	IMX_GPIO_NR(3, 1),	/* CON1_4, CON2_40 */
	IMX_GPIO_NR(3, 2),	/* CON1_5, CON2_39 */
	IMX_GPIO_NR(3, 3),	/* CON1_6, CON2_38 */
	IMX_GPIO_NR(4, 2),	/* CON1_7, CON2_32 */
	IMX_GPIO_NR(4, 3),	/* CON1_8, CON2_34 */
	IMX_GPIO_NR(4, 4),	/* CON1_9, CON2_36 */
	IMX_GPIO_NR(4, 5),	/* CON1_10, CON2_37 */
	IMX_GPIO_NR(4, 6),	/* CON1_11, CON2_35 */
	IMX_GPIO_NR(4, 7),	/* CON1_12, CON2_50 */
	IMX_GPIO_NR(4, 10),	/* CON1_13, CON2_24 */
	IMX_GPIO_NR(4, 11),	/* CON1_14, CON2_25 */
	IMX_GPIO_NR(5, 11),	/* CON1_15, CON2_33 */
	IMX_GPIO_NR(6, 19),	/* CON1_16, CON2_46 */
	IMX_GPIO_NR(6, 20),	/* CON1_17, CON2_47 */
	IMX_GPIO_NR(6, 22),	/* CON1_18, CON2_49 */
	IMX_GPIO_NR(6, 21),	/* CON1_19, CON2_48 */
	IMX_GPIO_NR(4, 12),	/* CON1_22, CON2_42 */
	IMX_GPIO_NR(4, 13),	/* CON1_23, CON2_43 */
	IMX_GPIO_NR(3, 25),	/* CON1_24 */
	IMX_GPIO_NR(3, 26),	/* CON1_25 */
	IMX_GPIO_NR(7, 12),	/* CON1_32 */
	IMX_GPIO_NR(7, 13),	/* CON1_33 */
	IMX_GPIO_NR(7, 7),	/* CON1_34 */
	IMX_GPIO_NR(7, 2),	/* CON1_35 */
	IMX_GPIO_NR(7, 8),	/* CON1_36 */
	IMX_GPIO_NR(7, 3),	/* CON1_37 */
	IMX_GPIO_NR(7, 0),	/* CON1_38 */
	IMX_GPIO_NR(7, 1),	/* CON1_39 */
	IMX_GPIO_NR(4, 1),	/* CON1_40 */
	IMX_GPIO_NR(4, 0),	/* CON1_41 */
	IMX_GPIO_NR(7, 4),	/* CON1_42 */
	IMX_GPIO_NR(7, 5),	/* CON1_43 */
	IMX_GPIO_NR(7, 6),	/* CON1_44 */
	IMX_GPIO_NR(7, 10),	/* CON1_45 */
	IMX_GPIO_NR(7, 11),	/* CON1_46 */
	IMX_GPIO_NR(5, 12),	/* CON1_47 */
	IMX_GPIO_NR(5, 13),	/* CON1_48 */
	IMX_GPIO_NR(5, 14),	/* CON1_49 */
	IMX_GPIO_NR(7, 9),	/* CON1_50 */
	IMX_GPIO_NR(5, 15),	/* CON1_51 */
	IMX_GPIO_NR(5, 16),	/* CON1_52 */
	IMX_GPIO_NR(5, 17),	/* CON1_53 */
};

#define IOX_SDI IMX_GPIO_NR(1, 9)
#define IOX_STCP IMX_GPIO_NR(1, 12)
#define IOX_SHCP IMX_GPIO_NR(1, 13)

static iomux_v3_cfg_t const misc_pads[] = {
	/* SIM1_PORT1_RD */
	MX7D_PAD_EPDC_DATA12__SIM1_PORT1_PD | MUX_PAD_CTRL(PAD_CTL_HYS),
	/* GPIO_3G_PWR_IND_3.3 */
	MX7D_PAD_ECSPI1_SCLK__GPIO4_IO16 | MUX_PAD_CTRL(0x0),
	/* PMIC_INT_B */
	MX7D_PAD_GPIO1_IO12__GPIO1_IO12	| MUX_PAD_CTRL(0x0),
};

#define MAINTENANCE_MODE_GPIO IMX_GPIO_NR(1, 9)

static iomux_v3_cfg_t const maintenance_mode_gpio_pads[] = {
	MX7D_PAD_GPIO1_IO09__GPIO1_IO9 | MUX_PAD_CTRL(NO_PAD_CTRL),
};

/*
 * PCIE_DIS_B --> Q0
 * PCIE_RST_B --> Q1
 * HDMI_RST_B --> Q2
 * PERI_RST_B --> Q3
 * SENSOR_RST_B --> Q4
 * ENET_RST_B --> Q5
 * PERI_3V3_EN --> Q6
 * LCD_PWR_EN --> Q7
 */
enum qn {
	PCIE_DIS_B,
	PCIE_RST_B,
	HDMI_RST_B,
	PERI_RST_B,
	SENSOR_RST_B,
	ENET_RST_B,
	PERI_3V3_EN,
	LCD_PWR_EN,
};

enum qn_func {
	qn_reset,
	qn_enable,
	qn_disable,
};

enum qn_level {
	qn_low = 0,
	qn_high = 1,
};

static enum qn_level seq[3][2] = {
	{0, 1}, {1, 1}, {0, 0}
};

static enum qn_func qn_output[8] = {
	qn_disable, qn_reset, qn_reset, qn_reset, qn_reset, qn_reset, qn_enable,
	qn_enable
};

void iox74lv_init(void)
{
	int i;

	for (i = 7; i >= 0; i--) {
		gpio_direction_output(IOX_SHCP, 0);
		gpio_direction_output(IOX_SDI, seq[qn_output[i]][0]);
		udelay(500);
		gpio_direction_output(IOX_SHCP, 1);
		udelay(500);
	}

	gpio_direction_output(IOX_STCP, 0);
	udelay(500);
	/*
	  * shift register will be output to pins
	  */
	gpio_direction_output(IOX_STCP, 1);

	for (i = 7; i >= 0; i--) {
		gpio_direction_output(IOX_SHCP, 0);
		gpio_direction_output(IOX_SDI, seq[qn_output[i]][1]);
		udelay(500);
		gpio_direction_output(IOX_SHCP, 1);
		udelay(500);
	}
	gpio_direction_output(IOX_STCP, 0);
	udelay(500);
	/*
	  * shift register will be output to pins
	  */
	gpio_direction_output(IOX_STCP, 1);
};

void iox74lv_set(int index)
{
	int i;
	for (i = 7; i >= 0; i--) {
		gpio_direction_output(IOX_SHCP, 0);

		if (i == index)
			gpio_direction_output(IOX_SDI, seq[qn_output[i]][0]);
		else
			gpio_direction_output(IOX_SDI, seq[qn_output[i]][1]);
		udelay(500);
		gpio_direction_output(IOX_SHCP, 1);
		udelay(500);
	}

	gpio_direction_output(IOX_STCP, 0);
	udelay(500);
	/*
	  * shift register will be output to pins
	  */
	gpio_direction_output(IOX_STCP, 1);

	for (i = 7; i >= 0; i--) {
		gpio_direction_output(IOX_SHCP, 0);
		gpio_direction_output(IOX_SDI, seq[qn_output[i]][1]);
		udelay(500);
		gpio_direction_output(IOX_SHCP, 1);
		udelay(500);
	}

	gpio_direction_output(IOX_STCP, 0);
	udelay(500);
	/*
	  * shift register will be output to pins
	  */
	gpio_direction_output(IOX_STCP, 1);
};

#ifdef CONFIG_FEC_MXC
static iomux_v3_cfg_t const fec2_pads[] = {
	MX7D_PAD_EPDC_SDCE0__ENET2_RGMII_RX_CTL | MUX_PAD_CTRL(ENET_RX_PAD_CTRL),
	MX7D_PAD_EPDC_SDCLK__ENET2_RGMII_RD0 | MUX_PAD_CTRL(ENET_RX_PAD_CTRL),
	MX7D_PAD_EPDC_SDLE__ENET2_RGMII_RD1 | MUX_PAD_CTRL(ENET_RX_PAD_CTRL),
	MX7D_PAD_EPDC_SDOE__ENET2_RGMII_RD2 | MUX_PAD_CTRL(ENET_RX_PAD_CTRL),
	MX7D_PAD_EPDC_SDSHR__ENET2_RGMII_RD3 | MUX_PAD_CTRL(ENET_RX_PAD_CTRL),
	MX7D_PAD_EPDC_SDCE1__ENET2_RGMII_RXC | MUX_PAD_CTRL(ENET_RX_PAD_CTRL),

	MX7D_PAD_EPDC_GDRL__ENET2_RGMII_TX_CTL | MUX_PAD_CTRL(ENET_PAD_CTRL),
	MX7D_PAD_EPDC_SDCE2__ENET2_RGMII_TD0 | MUX_PAD_CTRL(ENET_PAD_CTRL),
	MX7D_PAD_EPDC_SDCE3__ENET2_RGMII_TD1 | MUX_PAD_CTRL(ENET_PAD_CTRL),
	MX7D_PAD_EPDC_GDCLK__ENET2_RGMII_TD2 | MUX_PAD_CTRL(ENET_PAD_CTRL),
	MX7D_PAD_EPDC_GDOE__ENET2_RGMII_TD3 | MUX_PAD_CTRL(ENET_PAD_CTRL),
	MX7D_PAD_EPDC_GDSP__ENET2_RGMII_TXC | MUX_PAD_CTRL(ENET_PAD_CTRL),

	MX7D_PAD_GPIO1_IO14__ENET2_MDIO | MUX_PAD_CTRL(ENET_PAD_CTRL_MII),
	MX7D_PAD_GPIO1_IO15__ENET2_MDC | MUX_PAD_CTRL(ENET_PAD_CTRL_MII),

	/* GPIO_LAN1_RST_N */
	MX7D_PAD_EPDC_BDR1__GPIO2_IO29 | MUX_PAD_CTRL(NO_PAD_CTRL),
	MX7D_PAD_EPDC_PWR_COM__GPIO2_IO30,	/* GPIO_LAN1_COMA_MODE */
};

static void setup_iomux_fec(void)
{
	imx_iomux_v3_setup_multiple_pads(fec2_pads, ARRAY_SIZE(fec2_pads));
}
#endif

static void setup_iomux_uart(void)
{
	imx_iomux_v3_setup_multiple_pads(uart5_pads, ARRAY_SIZE(uart5_pads));
}

#ifdef CONFIG_QSPI
static iomux_v3_cfg_t const quadspi_pads[] = {
	MX7D_PAD_EPDC_DATA00__QSPI_A_DATA0 | MUX_PAD_CTRL(QSPI_PAD_CTRL),
	MX7D_PAD_EPDC_DATA01__QSPI_A_DATA1 | MUX_PAD_CTRL(QSPI_PAD_CTRL),
	MX7D_PAD_EPDC_DATA02__QSPI_A_DATA2 | MUX_PAD_CTRL(QSPI_PAD_CTRL),
	MX7D_PAD_EPDC_DATA03__QSPI_A_DATA3 | MUX_PAD_CTRL(QSPI_PAD_CTRL),
	MX7D_PAD_EPDC_DATA05__QSPI_A_SCLK  | MUX_PAD_CTRL(QSPI_PAD_CTRL),
	MX7D_PAD_EPDC_DATA06__QSPI_A_SS0_B | MUX_PAD_CTRL(QSPI_PAD_CTRL),
};

int board_qspi_init(void)
{
	/* Set the iomux */
	imx_iomux_v3_setup_multiple_pads(quadspi_pads, ARRAY_SIZE(quadspi_pads));

	/* Set the clock */
	set_clk_qspi();

	return 0;
}
#endif

#ifdef CONFIG_FSL_ESDHC

#define USDHC1_CD_GPIO	IMX_GPIO_NR(5, 0)
#define USDHC1_PWR_GPIO	IMX_GPIO_NR(5, 2)
#define USDHC3_PWR_GPIO IMX_GPIO_NR(6, 11)

static struct fsl_esdhc_cfg usdhc_cfg[3] = {
	{USDHC1_BASE_ADDR, 0, 4},
	{USDHC3_BASE_ADDR},
};

int mmc_get_env_devno(void)
{
	struct bootrom_sw_info **p =
		(struct bootrom_sw_info **)ROM_SW_INFO_ADDR;

	u8 boot_type = (*p)->boot_dev_type;
	u8 dev_no = (*p)->boot_dev_instance;

	/* If not boot from sd/mmc, use default value */
	if ((boot_type != BOOT_TYPE_SD) && (boot_type != BOOT_TYPE_MMC))
		return CONFIG_SYS_MMC_ENV_DEV;

	if (2 == dev_no)
		dev_no--;

	return dev_no;
}

int mmc_map_to_kernel_blk(int dev_no)
{
	if (1 == dev_no)
		dev_no++;

	return dev_no;
}

int board_mmc_getcd(struct mmc *mmc)
{
	struct fsl_esdhc_cfg *cfg = (struct fsl_esdhc_cfg *)mmc->priv;
	int ret = 0;

	switch (cfg->esdhc_base) {
	case USDHC1_BASE_ADDR:
		ret = !gpio_get_value(USDHC1_CD_GPIO);
		break;
	case USDHC3_BASE_ADDR:
		ret = 1; /* Assume uSDHC3 emmc is always present */
		break;
	}

	return ret;
}

int board_mmc_init(bd_t *bis)
{
	int i, ret;
	/*
	 * According to the board_mmc_init() the following map is done:
	 * (U-boot device node)    (Physical Port)
	 * mmc0                    USDHC1
	 * mmc2                    USDHC3 (eMMC)
	 */
	for (i = 0; i < CONFIG_SYS_FSL_USDHC_NUM; i++) {
		switch (i) {
		case 0:
			imx_iomux_v3_setup_multiple_pads(
				usdhc1_pads, ARRAY_SIZE(usdhc1_pads));
			gpio_request(USDHC1_CD_GPIO, "usdhc1_cd");
			gpio_direction_input(USDHC1_CD_GPIO);
			gpio_request(USDHC1_PWR_GPIO, "usdhc1_pwr");
			gpio_direction_output(USDHC1_PWR_GPIO, 0);
			udelay(500);
			gpio_direction_output(USDHC1_PWR_GPIO, 1);
			usdhc_cfg[0].sdhc_clk = mxc_get_clock(MXC_ESDHC_CLK);
			break;
		case 1:
			imx_iomux_v3_setup_multiple_pads(
				usdhc3_emmc_pads, ARRAY_SIZE(usdhc3_emmc_pads));
			gpio_request(USDHC3_PWR_GPIO, "usdhc3_pwr");
			gpio_direction_output(USDHC3_PWR_GPIO, 0);
			mdelay(1);
			gpio_direction_output(USDHC3_PWR_GPIO, 1);
			usdhc_cfg[1].sdhc_clk = mxc_get_clock(MXC_ESDHC3_CLK);
			break;
		default:
			printf("Warning: you configured more USDHC controllers"
				"(%d) than supported by the board\n", i + 1);
			return 0;
			}

			ret = fsl_esdhc_initialize(bis, &usdhc_cfg[i]);
			if (ret)
				return ret;
	}

	return 0;
}

int check_mmc_autodetect(void)
{
	char *autodetect_str = getenv("mmcautodetect");

	if ((autodetect_str != NULL) &&
		(strcmp(autodetect_str, "yes") == 0)) {
		return 1;
	}

	return 0;
}

#endif

#ifdef CONFIG_FEC_MXC
int board_eth_init(bd_t *bis)
{
	int ret;

	setup_iomux_fec();

	ret = fecmxc_initialize_multi(bis, 0,
		CONFIG_FEC_MXC_PHYADDR, IMX_FEC_BASE);
	if (ret)
		printf("FEC2 MXC: %s:failed\n", __func__);

	return ret;
}

static int setup_fec(void)
{
	struct iomuxc_gpr_base_regs *const iomuxc_gpr_regs
		= (struct iomuxc_gpr_base_regs *) IOMUXC_GPR_BASE_ADDR;
	int ret;

	/* Use 125M anatop REF_CLK1 for ENET1, clear gpr1[13], gpr1[17]*/
	clrsetbits_le32(&iomuxc_gpr_regs->gpr[1],
		(IOMUXC_GPR_GPR1_GPR_ENET1_TX_CLK_SEL_MASK |
		 IOMUXC_GPR_GPR1_GPR_ENET1_CLK_DIR_MASK), 0);

	ret = set_clk_enet(ENET_125MHz);
	if (ret)
		return ret;

	return 0;
}

#define VSC_PHY_MAIN_PAGE	(0x00)
#define VSC_PHY_EXT1_PAGE	(0x01)
#define VSC_PHY_EXT2_PAGE	(0x02)
#define VSC_PHY_EXT3_PAGE	(0x03)
#define VSC_PHY_EXT4_PAGE	(0x04)
#define VSC_PHY_GPIO_PAGE	(0x10)
#define VSC_PHY_TEST_PAGE	(0x2A30)
#define VSC_PHY_TR_PAGE		(0x52B5)

static int vsc_phy_read(struct phy_device *phydev, int page,
			int regnum)
{
	int ret;

	/* set access page */
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1f, page);

	/* read val */
	ret = phy_read(phydev, MDIO_DEVAD_NONE, regnum);

	if (page != VSC_PHY_MAIN_PAGE) {
		phy_write(phydev, MDIO_DEVAD_NONE, 0x1f,
			  VSC_PHY_MAIN_PAGE);
	}

	return ret;
}

static int vsc_phy_write(struct phy_device *phydev, int page,
			 int regnum, u16 val)
{
	int ret;

	/* set access page */
	phy_write(phydev, MDIO_DEVAD_NONE, 0x1f, page);

	/* write val */
	ret = phy_write(phydev, MDIO_DEVAD_NONE, regnum, val);

	if (page != VSC_PHY_MAIN_PAGE) {
		phy_write(phydev, MDIO_DEVAD_NONE, 0x1f,
			  VSC_PHY_MAIN_PAGE);
	}

	return ret;
}

static int vsc_phy_write_masked(struct phy_device *phydev, int page,
				int regnum, u16 val, u16 mask)
{
	int tmp;

	tmp = vsc_phy_read(phydev, page, regnum);
	tmp &= ~mask;
	tmp |= (val & mask);

	return vsc_phy_write(phydev, page, regnum, tmp);
}

static const u16 init_script_nano_rev_a[] = {
//     Op,   Page,  Reg, Value,  Mask
//      0,      1,    2,     3,     4
//     --, ------, ----, ------, -----
        0, 0x0000, 0x1f, 0x0000, 0xffff,
        1, 0x0000, 0x16, 0x0001, 0x0001,
        0, 0x0001, 0x1f, 0x2A30, 0xffff,
        1, 0x2A30, 0x18, 0x0000, 0x0400,
        1, 0x2A30, 0x05, 0x0c00, 0x0e00,
        1, 0x2A30, 0x08, 0x8000, 0x8000,
        0, 0x2A30, 0x1f, 0x52B5, 0xffff,
        0, 0x52B5, 0x12, 0x0068, 0xffff,
        0, 0x52B5, 0x11, 0x8980, 0xffff,
        0, 0x52B5, 0x10, 0x8f90, 0xffff,
        0, 0x52B5, 0x12, 0x0000, 0xffff,
        0, 0x52B5, 0x11, 0x0003, 0xffff,
        0, 0x52B5, 0x10, 0x8796, 0xffff,
        0, 0x52B5, 0x12, 0x0050, 0xffff,
        0, 0x52B5, 0x11, 0x100f, 0xffff,
        0, 0x52B5, 0x10, 0x87fa, 0xffff,
        0, 0x52B5, 0x1f, 0x2A30, 0xffff,
        1, 0x2A30, 0x08, 0x0000, 0x8000,
        0, 0x2A30, 0x1f, 0x0000, 0xffff,
        1, 0x0000, 0x16, 0x0000, 0x0001,
	0xf, 0xffff, 0xff, 0xffff, 0xffff,
};

static int load_init_script(struct phy_device *phydev)
{
	u16 op, page, reg, value, mask;
	int i;

	for (i = 0; i < ARRAY_SIZE(init_script_nano_rev_a);) {
		op    = init_script_nano_rev_a[i++];
		page  = init_script_nano_rev_a[i++];
		reg   = init_script_nano_rev_a[i++];
		value = init_script_nano_rev_a[i++];
		mask  = init_script_nano_rev_a[i++];

		if ((op == 0) || (op == 1)) {
			vsc_phy_write_masked(phydev, page, reg,
					     value, mask);
		}
	}

	return 0;
}

static int apply_nano_patch(struct phy_device *phydev)
{
	int val;

	val = vsc_phy_read(phydev, VSC_PHY_EXT1_PAGE, 0x17);
	if ((val & 0xf800) >> 11 != 0) {
		printf("unsupported port %d\n", val);
		return -1;
	}

	return load_init_script(phydev);
}

#define VSC8501_RST_N		IMX_GPIO_NR(2, 29)
#define VSC8501_COMA_MODE	IMX_GPIO_NR(2, 30)
int board_phy_config(struct phy_device *phydev)
{
	int ret;

	/* enable rgmii rxc/gtxc skew and phy mode select to RGMII copper */

	gpio_request(VSC8501_COMA_MODE, "lan1_coma_mode");

	/* Reset active, drive low. */
	gpio_direction_output(VSC8501_RST_N, 0);
	mdelay(2);

	/* 1. COMA_MODE active, drive high. */
	gpio_direction_output(VSC8501_COMA_MODE, 1);
	mdelay(10);

	/* 2. Apply power. */
	/* 3. Apply RefClk. */

	/* 4. Release reset, drive high. */
	gpio_direction_output(VSC8501_RST_N, 1);
	mdelay(105);

	/* 5. Wait 120 ms minimum. */
	mdelay(120);

	/* 6. Apply patch from PHY_API. */
	ret = apply_nano_patch(phydev);
	if (ret)
		return ret;

	/* 7. Configure register 23 for MAC interface mode. */
	/* 8. Write new register 23. */
	vsc_phy_write_masked(phydev, VSC_PHY_MAIN_PAGE, 0x17,
			     0x3000, 0x3000);	/* RGMII */

	/* 9. Software reset. */
	/* 10. Write new register 0. */
	/* 11. Read register 0 untio bit 15 equals to 0. */
	ret = phy_reset(phydev);
	if (ret)
		return ret;

	/* 12. For RGMII mode: Configure register 20E2. */
	vsc_phy_write_masked(phydev, VSC_PHY_EXT2_PAGE, 0x14,
			     0x0000, 0x0800);
	vsc_phy_write_masked(phydev, VSC_PHY_EXT2_PAGE, 0x14,
			     0x0033, 0x0077);

	/* 13. Release the COMA_MODE pin. */
	gpio_direction_output(VSC8501_COMA_MODE, 0);

	/* LED configuration */
	vsc_phy_write_masked(phydev, VSC_PHY_MAIN_PAGE, 0x1d,
			     0x0ee0, 0x0fff);
	vsc_phy_write_masked(phydev, VSC_PHY_GPIO_PAGE, 0x0e,
			     0x0000, 0x0200);
	vsc_phy_write_masked(phydev, VSC_PHY_GPIO_PAGE, 0x1e,
			     0x0800, 0x0c00);

	if (phydev->drv->config)
		phydev->drv->config(phydev);
	return 0;
}
#endif

void setup_a7_clk(void)
{
	u32 target;

	/* set enet axi clock 792M: 792/1 */
	target = CLK_ROOT_ON | ARM_A7_CLK_ROOT_FROM_PLL_ARM_MAIN_800M_CLK |
		 CLK_ROOT_PRE_DIV(CLK_ROOT_PRE_DIV1) |
		 CLK_ROOT_POST_DIV(CLK_ROOT_POST_DIV1);
	clock_set_target_val(ARM_A7_CLK_ROOT, target);

	/* set AXI clock 332M: 332/1*/
	target = CLK_ROOT_ON | MAIN_AXI_CLK_ROOT_FROM_PLL_SYS_PFD1_332M_CLK |
		CLK_ROOT_PRE_DIV(CLK_ROOT_PRE_DIV1) |
		CLK_ROOT_POST_DIV(CLK_ROOT_POST_DIV1);
	clock_set_target_val(MAIN_AXI_CLK_ROOT, target);
}

#define BMIC_I2C_BUS		3
#define BMIC_I2C_GENERAL_SLAVE	0x10
#define BMIC_I2C_EEPROM_SLAVE	0x15
#define BMIC_I2C_EEPROM_VER	0x01
#define BMIC_I2C_EEPROM_DATA	0x02
#define BMIC_I2C_EEPROM_SIZE	1024

/* wakeup when bmic is sleep mode. */
static void bmic_init(void)
{
	i2c_set_bus_num(BMIC_I2C_BUS);

	i2c_init(CONFIG_SYS_I2C_SPEED, BMIC_I2C_GENERAL_SLAVE);

	i2c_probe(BMIC_I2C_GENERAL_SLAVE);
	mdelay(20);
}

static int bmic_eeprom_init(void)
{
	i2c_set_bus_num(BMIC_I2C_BUS);

	i2c_init(CONFIG_SYS_I2C_SPEED, BMIC_I2C_EEPROM_SLAVE);

	return i2c_probe(BMIC_I2C_EEPROM_SLAVE);
}

static int bmic_eeprom_read_byte(u16 addr, u8 *data)
{
	if (i2c_write(BMIC_I2C_EEPROM_SLAVE, BMIC_I2C_EEPROM_DATA, 1, (u8 *)&addr, sizeof(addr))) {
		printf("bmic eeprom read error! addr = %d: Command Write\n", addr);
		return -1;
	}

	if (i2c_read(BMIC_I2C_EEPROM_SLAVE, 0, 0, data, 1)) {
		printf("bmic eeprom read error! addr = %d: Data Read\n", addr);
		return -1;
	}

	return 0;
}

static int bmic_eeprom_read(u16 addr, void *data, u16 size)
{
	u8 *pdata = (u8 *)data;

	if ((addr + size) > BMIC_I2C_EEPROM_SIZE) {
		printf("bmic eeprom read error! Size \n");
		return -1;
	}

	while (size--) {
		if (bmic_eeprom_read_byte(addr, pdata) != 0)
			return -1;
		addr++;
		pdata++;
	}

	return 0;
}

int board_info_init(void)
{
	u8 serial_no_vaild;

	memcpy(&x1_board_info, 0x00, sizeof(x1_board_info));

	bmic_init();

	if (bmic_eeprom_init()) {
		printf("error: BMIC EEPROM detect\n");
		x1_board_info.board_type = BOARDTYPE_AIOTG_G3;
		x1_board_info.revison = 0x0000;
		return -1;
	}

	bmic_eeprom_read(0, &x1_board_info.board_type, sizeof(x1_board_info.board_type));
	bmic_eeprom_read(4, &x1_board_info.revison, sizeof(x1_board_info.revison));
	bmic_eeprom_read(15, &serial_no_vaild, sizeof(serial_no_vaild));
	if (serial_no_vaild)
		bmic_eeprom_read(8, &x1_board_info.serial_no, sizeof(x1_board_info.serial_no));
	bmic_eeprom_read(16, &x1_board_info.dram, sizeof(x1_board_info.dram));
	bmic_eeprom_read(20, &x1_board_info.xtal, sizeof(x1_board_info.xtal));
	bmic_eeprom_read(32, &x1_board_info.usermac, 6);

	puts("Board Type: ");
	switch (x1_board_info.board_type) {
	case BOARDTYPE_AX1:
		puts("Armadillo-X1");
		break;
	case BOARDTYPE_AIOTG_G3L:
		puts("Armadillo-IoT G3L");
		break;
	case BOARDTYPE_AIOTG_G3:
	default:
		puts("Armadillo-IoT G3");
		break;
	}
	printf("(%08x)\n", x1_board_info.board_type);

	printf("Revison: %04x\n", x1_board_info.revison);
	printf("S/N: %d\n", (int)x1_board_info.serial_no);
	printf("DRAM: %08x\n", x1_board_info.dram);
	printf("XTAL: %02x\n", x1_board_info.xtal);

	return 0;
}

int board_early_init_f(void)
{
	setup_a7_clk();
	setup_iomux_uart();

#ifdef CONFIG_SYS_I2C_MXC
	setup_i2c(3, CONFIG_SYS_I2C_SPEED, 0x7f, &i2c_pad_info4);
#endif

	return 0;
}

int board_init(void)
{
	/* address of boot parameters */
	gd->bd->bi_boot_params = PHYS_SDRAM + 0x100;

	imx_iomux_v3_setup_multiple_pads(misc_pads, ARRAY_SIZE(misc_pads));

	iox74lv_init();

	gpio_request(VSC8501_RST_N, "lan1_rst_n");
	gpio_direction_output(VSC8501_RST_N, 1);

#ifdef CONFIG_FEC_MXC
	setup_fec();
#endif

#ifdef CONFIG_QSPI
	board_qspi_init();
#endif
	return 0;
}

#ifdef CONFIG_CMD_BMODE
static const struct boot_mode board_boot_modes[] = {
	/* 4 bit bus width */
	{"sd1", MAKE_CFGVAL(0x10, 0x10, 0x00, 0x00)},
	{"emmc", MAKE_CFGVAL(0x10, 0x2a, 0x00, 0x00)},
	/* TODO: Nand */
	{"qspi", MAKE_CFGVAL(0x00, 0x40, 0x00, 0x00)},
	{NULL,   0},
};
#endif

#ifdef CONFIG_PFUZE3000_PMIC_I2C
#define PFUZE_DEVICEID	0x0
#define PFUZE_REVID	0x3
#define PFUZE_FABID	0x4

#define PFUZE_LDOGCTL	0x69

#define USB_HUB_RESET_N		IMX_GPIO_NR(2, 13)
#define PMIC_I2C_SLAVE_DEFAULT	0x08
#define PFUZE_PAGE		0x7f
#define 	PFUZE_PAGE_FUNCTIONAL_PAGE	0x0
#define 	PFUZE_PAGE_EXTENDED_PAGE1	0x1
#define 	PFUZE_PAGE_EXTENDED_PAGE2	0x2
#define PFUZE_OTP_I2C_ADDR	0xff
#define 	PFUZE_OTP_I2C_ADDR_USE_DEFAULT	(1<<7)
#define 	PFUZE_OTP_I2C_ADDR_SLB_ADDR_MASK	(0x03)
void siot_workaround(void)
{
	unsigned char value;

	/* assert USB Hub reset */
	imx_iomux_v3_setup_pad(MX7D_PAD_EPDC_DATA13__GPIO2_IO13);
	gpio_direction_output(USB_HUB_RESET_N, 0);

	i2c_set_bus_num(CONFIG_PMIC_I2C_BUS);

	i2c_init(CONFIG_SYS_I2C_SPEED, PMIC_I2C_SLAVE_DEFAULT);
	if (!i2c_probe(PMIC_I2C_SLAVE_DEFAULT)) {
		value = PFUZE_PAGE_EXTENDED_PAGE1;
		if (i2c_write(PMIC_I2C_SLAVE_DEFAULT, PFUZE_PAGE, 1, &value, 1)) {
			printf("Write Extended Page1 error!\n");
			return;
		}

		if (i2c_read(PMIC_I2C_SLAVE_DEFAULT, PFUZE_OTP_I2C_ADDR, 1, &value, 1)) {
			printf("Read I2C address error!\n");
			return;
		}
		if (value & PFUZE_OTP_I2C_ADDR_USE_DEFAULT)
			return;

		value |= (CONFIG_PMIC_I2C_SLAVE - PMIC_I2C_SLAVE_DEFAULT) &
			PFUZE_OTP_I2C_ADDR_SLB_ADDR_MASK;
		if (i2c_write(PMIC_I2C_SLAVE_DEFAULT, PFUZE_OTP_I2C_ADDR, 1, &value, 1)) {
			printf("Write I2C address error!\n");
			return;
		}

		value = PFUZE_PAGE_FUNCTIONAL_PAGE;
		if (i2c_write(CONFIG_PMIC_I2C_SLAVE, PFUZE_PAGE, 1, &value, 1)) {
			printf("Write Default Page error!\n");
			return;
		}
	}
}

static int setup_pmic_voltages(void)
{
	unsigned char value, rev_id = 0;

	siot_workaround();

	i2c_set_bus_num(CONFIG_PMIC_I2C_BUS);

	i2c_init(CONFIG_SYS_I2C_SPEED, CONFIG_PMIC_I2C_SLAVE);
	if (!i2c_probe(CONFIG_PMIC_I2C_SLAVE)) {
		if (i2c_read(CONFIG_PMIC_I2C_SLAVE, PFUZE_DEVICEID, 1, &value, 1)) {
			printf("Read device ID error!\n");
			return -1;
		}
		if (i2c_read(CONFIG_PMIC_I2C_SLAVE, PFUZE_REVID, 1, &rev_id, 1)) {
			printf("Read Rev ID error!\n");
			return -1;
		}
		printf("Found PFUZE300! deviceid 0x%x, revid 0x%x\n", value, rev_id);

		/* disable Low Power Mode during standby mode */
		if (i2c_read(CONFIG_PMIC_I2C_SLAVE, PFUZE_LDOGCTL, 1, &value, 1)) {
			printf("Read LDOCTL error!\n");
			return -1;
		}
		value |= 0x1;
		if (i2c_write(CONFIG_PMIC_I2C_SLAVE, PFUZE_LDOGCTL, 1, &value, 1)) {
			printf("Set LDOCTL error!\n");
			return -1;
		}
	}

	return 0;
}
#endif

#define AIOT_G3_LED3	IMX_GPIO_NR(3, 11)
#define AIOT_G3_LED2	IMX_GPIO_NR(3, 10)
#define AIOT_G3_LED4	IMX_GPIO_NR(3, 12)
#define AIOT_G3_LED5	IMX_GPIO_NR(3, 13)

static int aiot_g3_led_init(void)
{
	imx_iomux_v3_setup_multiple_pads(gpio_aiot_g3_led_pads,
					 ARRAY_SIZE(gpio_aiot_g3_led_pads));

	gpio_direction_output(AIOT_G3_LED3, 1);
	gpio_direction_output(AIOT_G3_LED2, 0);
	gpio_direction_output(AIOT_G3_LED4, 0);
	gpio_direction_output(AIOT_G3_LED5, 0);

	return 0;

}

static int aiot_g3_addon(void)
{
	int i;

	imx_iomux_v3_setup_multiple_pads(gpio_aiot_g3_addon_pads,
					 ARRAY_SIZE(gpio_aiot_g3_addon_pads));
	for (i = 0; i < ARRAY_SIZE(gpio_aiot_g3_addon); i++)
		gpio_direction_input(gpio_aiot_g3_addon[i]);

	return 0;
}

int board_late_init(void)
{
#ifdef CONFIG_CMD_BMODE
	add_board_boot_modes(board_boot_modes);
#endif

#ifdef CONFIG_PFUZE3000_PMIC_I2C
	int ret = 0;

	ret = setup_pmic_voltages();
	if (ret)
		return ret;
#endif

	imx_iomux_v3_setup_multiple_pads(wdog_pads, ARRAY_SIZE(wdog_pads));

	set_wdog_reset((struct wdog_regs *)WDOG1_BASE_ADDR);

	imx_iomux_v3_setup_multiple_pads(maintenance_mode_gpio_pads,
					 ARRAY_SIZE(maintenance_mode_gpio_pads));
	gpio_direction_input(MAINTENANCE_MODE_GPIO);

	board_info_init();

	switch (x1_board_info.board_type) {
	case BOARDTYPE_AX1:
		break;
	case BOARDTYPE_AIOTG_G3L:
		break;
	case BOARDTYPE_AIOTG_G3:
	default:
		aiot_g3_led_init();
		aiot_g3_addon();
		break;
	}

	hw_watchdog_init();

	return 0;
}

u32 get_board_rev(void)
{
	return get_cpu_rev();
}

int checkboard(void)
{
	return 0;
}

#ifdef CONFIG_USB_EHCI_MX7
iomux_v3_cfg_t const usb_otg1_pads[] = {
	MX7D_PAD_GPIO1_IO05__USB_OTG1_PWR | MUX_PAD_CTRL(NO_PAD_CTRL),
};

int board_ehci_hcd_init(int port)
{
	switch (port) {
	case 0:
		imx_iomux_v3_setup_multiple_pads(usb_otg1_pads,
						 ARRAY_SIZE(usb_otg1_pads));
		break;
	default:
		printf("MXC USB port %d not yet supported\n", port);
		return 1;
	}
	return 0;
}
#endif

#ifdef CONFIG_FASTBOOT

void board_fastboot_setup(void)
{
	switch (get_boot_device()) {
#if defined(CONFIG_FASTBOOT_STORAGE_MMC)
	case SD1_BOOT:
	case MMC1_BOOT:
		if (!getenv("fastboot_dev"))
			setenv("fastboot_dev", "mmc0");
		if (!getenv("bootcmd"))
			setenv("bootcmd", "booti mmc0");
		break;
	case SD3_BOOT:
	case MMC3_BOOT:
		if (!getenv("fastboot_dev"))
			setenv("fastboot_dev", "mmc1");
		if (!getenv("bootcmd"))
			setenv("bootcmd", "booti mmc1");
		break;
#endif /*CONFIG_FASTBOOT_STORAGE_MMC*/
	default:
		printf("unsupported boot devices\n");
		break;
	}
}

#endif /*CONFIG_FASTBOOT*/

#ifdef CONFIG_IMX_UDC
void udc_pins_setting(void)
{
}

#endif /*CONFIG_IMX_UDC*/

#ifdef CONFIG_SUPPORT_MAINTENANCE_MODE
int is_maintenance_mode(void)
{
	return !gpio_get_value(MAINTENANCE_MODE_GPIO);
}
#endif /* CONFIG_SUPPORT_MAINTENANCE_MODE */
